unit svcGuiService;

interface



uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    Graphics,
    Controls,
    SvCom_NTService,
    SvCom_ComService,
    Dialogs, 
    SvCom_LaunchFrontEnd,
    SvCom_WtsSessions,
    SvCom_WtsApi, 
    SvCom_ProcessList,
    SvCom_Timer;

type
  TGuiService = class(TComService)
    svWtsSessions: TsvWtsSessions;
    svLaunchFrontEnd: TsvLaunchFrontEnd;
    svTimer: TsvTimer;
    procedure svWtsSessionsSessionStateChanged(Sender: TObject;
      Session: TsvWtsSessionInfo);
    procedure ComServiceCreate(Sender: TObject);
    procedure svLaunchFrontEndProcessTerminate(Sender: TObject;
      Process: TsvLaunchedProcess);
    procedure svTimerTimer(Sender: TObject);
    procedure svLaunchFrontEndProcessLaunch(Sender: TObject;
      Process: TsvLaunchedProcess);
    procedure ComServiceStart(Sender: TNtService; var DoAction: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


var
  GuiService: TGuiService;

implementation

{$R *.DFM}

procedure TGuiService.svWtsSessionsSessionStateChanged(Sender: TObject;
  Session: TsvWtsSessionInfo);
var i: Integer;
    LP: TsvLaunchedProcess;
begin
    EventLog.LogMessage(Format('Session state changed : SessionId = %d  State = %d SID=%s',[Session.SessionId, Integer(Session.State), svWtsSessions.LogonSid]));

    // Check whether the user is active
    // Exit if it is not
    if Session.State <> WTSActive then exit;

    // Look up for already launched process
    for i := svLaunchFrontEnd.LaunchedProcessCount-1 downto 0 do
    begin
        LP:=svLaunchFrontEnd.LaunchedProcess[i];
        if not LP.IsAlive then
        begin
            LP.Free; // We should cleanup LaunchedProcess entries for terminated processes
            continue;
        end;
        if LP.SessionId = Session.SessionId then exit; // Yes, it still runs
    end;

    EventLog.LogMessage('Launching...');
    svLaunchFrontEnd.Launch(Session.SessionId);
    EventLog.LogMessage('Launched');
end;

procedure TGuiService.ComServiceCreate(Sender: TObject);
var S: String;
begin
    S:=ExtractFilePath(ParamStr(0));
    SetLength(S,Length(S)-1);
    S:=ExtractFilePath(S)+'011-GuiClient\GuiClient.exe';
    svLaunchFrontEnd.ApplicationName:=S;
end;

procedure TGuiService.svLaunchFrontEndProcessTerminate(Sender: TObject;
  Process: TsvLaunchedProcess);
var Session: TsvWtsSessionInfo;
begin
    svWtsSessions.UpdateSessions;

    svWtsSessions.Lock;
    try
        Session:=svWtsSessions.SessionsById[Process.SessionId];

        if not Assigned(Session) or (Session.State <> WTSActive) then
        begin
            Process.Free; // We should cleanup LaunchedProcess entries if we do not need it more
            exit;
        end;
    finally
        svWtsSessions.Unlock;
    end;

    svLaunchFrontEnd.Launch(Process.SessionId);
end;

procedure TGuiService.svTimerTimer(Sender: TObject);
begin
    svWtsSessions.UpdateSessions;
end;

procedure TGuiService.svLaunchFrontEndProcessLaunch(Sender: TObject;
  Process: TsvLaunchedProcess);
begin
    EventLog.LogMessage('It is really launched!');
end;

procedure TGuiService.ComServiceStart(Sender: TNtService;
  var DoAction: Boolean);
var i: Integer;
    Session: TsvWtsSessionInfo;
begin
    svWtsSessions.UpdateSessions;

    svWtsSessions.Lock;
    try
        for i:=0 to svWtsSessions.SessionCount-1 do
        begin
            Session:=svWtsSessions.Sessions[i];
            if (Session.State <> WTSActive) then continue;
            svLaunchFrontEnd.Launch(Session.SessionId);
        end;
    finally
        svWtsSessions.Unlock;
    end;
end;

end.
